﻿using AutoMapper;
using Correo.Domain.Models;
using Correo.PublishedLanguage.Commands;
using Correo.PublishedLanguage.Events;
using MediatR;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using NBB.Messaging.Abstractions;
using System;
using System.Threading;
using System.Threading.Tasks;

namespace Correo.Application.CommandHandlers
{
    public class SendEmailHandler : IRequestHandler<SendEmail>
    {
        private readonly IMessageBusPublisher _messageBusPublisher;
        private readonly ILogger<SendEmailHandler> _logger;
        private readonly IMapper _mapper;
        private readonly IOptions<DefaultSender> _defaultSender;

        public SendEmailHandler(IMessageBusPublisher messageBusPublisher, ILogger<SendEmailHandler> logger, IMapper mapper, IOptions<DefaultSender> defaultSender)
        {
            _messageBusPublisher=messageBusPublisher;
            _logger=logger;
            _mapper=mapper;
            _defaultSender=defaultSender;
        }

        public async Task<Unit> Handle(SendEmail command, CancellationToken cancellationToken)
        {
            try
            {
                var emailMessage = _mapper.Map<EmailMessage>(command);
                if (emailMessage.From == null)
                    emailMessage.From = new EmailMessage.MailAddress(_defaultSender.Value.Address, _defaultSender.Value.Name);

                // send email
            }
            catch (Exception ex)
            {
                await _messageBusPublisher.PublishAsync(new EmailSentFailed(command.Subject, ex.Message), cancellationToken);
                throw;
            }

            return Unit.Value;
        }
    }
}
